﻿
local directory = "Interface\\AddOns\\CritSound\\Sounds\\"
local sounds = {
	--spell damage
	spell = {
			"CriticalDamage.mp3",	--first time
			"Dominating.mp3",		--second time
			"Unstoppable.mp3",		--etc.
			"Wickedsick.mp3",
			"Godlike.mp3",
		},
	--magic shield
	shield = {
			"Shield.mp3",
		},
	--melee damage
	swing = {
			"Fight.mp3",
			"Swing.mp3",
			"SoulShatter.mp3",
		},
	--range damage, such as shooting, wand
	range = {
			"Gun1.mp3",
			"Gun2.mp3",
			"Gun3.mp3",
		},
	--healing spells
	heal = {
			"CritHeal.mp3",
		}
}
local records = {
	spell	= {times = 0, lasttime = 0},
	shield	= {times = 0, lasttime = 0},
	swing	= {times = 0, lasttime = 0},
	range	= {times = 0, lasttime = 0},
	heal	= {times = 0, lasttime = 0}
}

if (AgingTime == nil) then 
	AgingTime = 10 --the aging time for cleaning up records.
end
if (SoundMode == nil) then 
	SoundMode = 2 --1: single mode, 2: sequence mode, 3: random mode.
end
if (SoundSwitch == nil) then 
	SoundSwitch = {spell = 1, shield = 1, swing = 1, range = 1, heal = 1}
end


function CritSound_OnLoad(self)
    self:RegisterEvent("VARIABLES_LOADED")
    self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

	--Slash Commands
	SlashCmdList["CritSound"] = CritSound_Command
	SLASH_CritSound1 = "/critsound"
	SLASH_CritSound2 = "/cs"
end

function CritSound_OnEvent(self, event, ...)
	if (event == "VARIABLES_LOADED") then
		print("|cFFFFFF99"..CRITSOUND_LOADED)
	end

	local timestamp, event, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags = ...
	if CombatLog_Object_IsA(sourceFlags, COMBATLOG_FILTER_MINE) then
		if (event == "SPELL_DAMAGE") then
			if (SoundSwitch["spell"] == 1) then
				critical = select(21, ...)
				if critical then
					CritSound_UpdateTable("spell")
					CritSound_Announce("spell")
				else
					CritSound_WipeTable("spell")
				end
			end
		elseif (event == "DAMAGE_SHIELD") then
			if (SoundSwitch["shield"] == 1) then
				critical = select(21, ...)
				if critical then
					CritSound_UpdateTable("shield")
					CritSound_Announce("shield")
				else
					CritSound_WipeTable("shield")
				end
			end
		elseif (event == "SWING_DAMAGE") then
			if (SoundSwitch["swing"] == 1) then
				critical = select(18, ...)
				if critical then
					CritSound_UpdateTable("swing")
					CritSound_Announce("swing")
				else
					CritSound_WipeTable("swing")
				end
			end
		elseif (event == "RANGE_DAMAGE") then
			if (SoundSwitch["range"] == 1) then
				critical = select(21, ...)
				if critical then
					CritSound_UpdateTable("range")
					CritSound_Announce("range")
				else
					CritSound_WipeTable("range")
				end
			end
		elseif (event == "SPELL_HEAL") then
			if (SoundSwitch["heal"] == 1) then
				critical = select(18, ...)
				if critical then
					CritSound_UpdateTable("heal")
					CritSound_Announce("heal")
				else
					CritSound_WipeTable("heal")
				end
			end
		end
	end
end

--clean up the records
function CritSound_WipeTable(tpye)
	records[tpye]["times"] = 0
end

--update the records when critical hits happen
function CritSound_UpdateTable(tpye)
	local timenow = GetTime()
	if (timenow - records[tpye]["lasttime"] < AgingTime) then
		records[tpye]["times"] = records[tpye]["times"] + 1
	else
		records[tpye]["times"] = 1
	end

	records[tpye]["lasttime"] = timenow
end

--play sounds when critical hits happen
function CritSound_Announce(tpye)
	local num = records[tpye]["times"]
	local len = table.getn(sounds[tpye])
	if (SoundMode == 1) then
		PlaySoundFile(directory..sounds[tpye][1])
	elseif (SoundMode == 2) then
		if (num <= len) then
			PlaySoundFile(directory..sounds[tpye][num])
		else
			PlaySoundFile(directory..sounds[tpye][len])
		end
	elseif (SoundMode == 3) then
		PlaySoundFile(directory..sounds[tpye][math.random(1,len)])
	end
end

--slash commands
function CritSound_Command(msg)
	local msg = string.lower(msg)
	if (not msg or msg == "" or msg == "help" or msg == "?") then
		print("|cFFFFFF99"..CRITSOUND_COMMANDS)
		if (SoundMode == 1) then
			print("|cFFFFFF99/cs mode |cFF00EEFFtype |cFFFFFF99"..CRITSOUND_SINGLE..CRITSOUND_HELP)
		elseif (SoundMode == 2) then
			print("|cFFFFFF99/cs mode |cFF00EEFFtype |cFFFFFF99"..CRITSOUND_SEQUENCE..CRITSOUND_HELP)
		elseif (SoundMode == 3) then
			print("|cFFFFFF99/cs mode |cFF00EEFFtype |cFFFFFF99"..CRITSOUND_RANDOM..CRITSOUND_HELP)
		end
		print("|cFFFFFF99/cs aging |cFF00EEFFtime |cFFFFFF99"..CRITSOUND_AGING..AgingTime..CRITSOUND_SECOND..CRITSOUND_AGINGERROR)
		if (SoundSwitch["spell"] == 1) then
			print("|cFFFFFF99/cs spell |cFF00EEFFon/off |cFFFFFF99"..CRITSOUND_SPELLON)
		else
			print("|cFFFFFF99/cs spell |cFF00EEFFon/off |cFFFFFF99"..CRITSOUND_SPELLOFF)
		end
		if (SoundSwitch["shield"] == 1) then
			print("|cFFFFFF99/cs shield |cFF00EEFFon/off |cFFFFFF99"..CRITSOUND_SHIELDON)
		else
			print("|cFFFFFF99/cs shield |cFF00EEFFon/off |cFFFFFF99"..CRITSOUND_SHIELDOFF)
		end
		if (SoundSwitch["swing"] == 1) then
			print("|cFFFFFF99/cs swing |cFF00EEFFon/off |cFFFFFF99"..CRITSOUND_SWINGON)
		else
			print("|cFFFFFF99/cs swing |cFF00EEFFon/off |cFFFFFF99"..CRITSOUND_SWINGOFF)
		end
		if (SoundSwitch["range"] == 1) then
			print("|cFFFFFF99/cs range |cFF00EEFFon/off |cFFFFFF99"..CRITSOUND_RANGEON)
		else
			print("|cFFFFFF99/cs range |cFF00EEFFon/off |cFFFFFF99"..CRITSOUND_RANGEOFF)
		end
		if (SoundSwitch["heal"] == 1) then
			print("|cFFFFFF99/cs heal |cFF00EEFFon/off |cFFFFFF99"..CRITSOUND_HEALON)
		else
			print("|cFFFFFF99/cs heal |cFF00EEFFon/off |cFFFFFF99"..CRITSOUND_HEALOFF)
		end
		return
	end

	local cmd, param = msg:match("^([%w%-]+)%s*(.*)$")
	if (cmd == "mode") then
		local modenumber = tonumber(param)
		if (modenumber == 1) then
			SoundMode = 1
			print("|cFFFFFF99"..CRITSOUND_SINGLE)
		elseif (modenumber == 2) then
			SoundMode = 2
			print("|cFFFFFF99"..CRITSOUND_SEQUENCE)
		elseif (modenumber == 3) then
			SoundMode = 3
			print("|cFFFFFF99"..CRITSOUND_RANDOM)
		else
			print("|cFFFFFF99"..CRITSOUND_MODEERROR)
		end
	elseif (cmd == "aging") then
		local agingtimes = tonumber(param)
		if (agingtimes > 0 and agingtimes <= 60) then
			AgingTime = agingtimes
			print("|cFFFFFF99"..CRITSOUND_AGING..AgingTime..CRITSOUND_SECOND)
		else
			print("|cFFFFFF99"..CRITSOUND_AGINGERROR)
		end
	elseif (cmd == "spell") then
		if (param == "on") then
			SoundSwitch["spell"] = 1
			print("|cFFFFFF99"..CRITSOUND_SPELLON)
		elseif (param == "off") then
			SoundSwitch["spell"] = 0
			print("|cFFFFFF99"..CRITSOUND_SPELLOFF)
		else
			print("|cFFFFFF99"..CRITSOUND_ONOFFERROR)
		end
	elseif (cmd == "shield") then
		if (param == "on") then
			SoundSwitch["shield"] = 1
			print("|cFFFFFF99"..CRITSOUND_SHIELDON)
		elseif (param == "off") then
			SoundSwitch["shield"] = 0
			print("|cFFFFFF99"..CRITSOUND_SHIELDOFF)
		else
			print("|cFFFFFF99"..CRITSOUND_ONOFFERROR)
		end
	elseif (cmd == "swing") then
		if (param == "on") then
			SoundSwitch["swing"] = 1
			print("|cFFFFFF99"..CRITSOUND_SWINGON)
		elseif (param == "off") then
			SoundSwitch["swing"] = 0
			print("|cFFFFFF99"..CRITSOUND_SWINGOFF)
		else
			print("|cFFFFFF99"..CRITSOUND_ONOFFERROR)
		end
	elseif (cmd == "range") then
		if (param == "on") then
			SoundSwitch["range"] = 1
			print("|cFFFFFF99"..CRITSOUND_RANGEON)
		elseif (param == "off") then
			SoundSwitch["range"] = 0
			print("|cFFFFFF99"..CRITSOUND_RANGEOFF)
		else
			print("|cFFFFFF99"..CRITSOUND_ONOFFERROR)
		end
	elseif (cmd == "heal") then
		if (param == "on") then
			SoundSwitch["heal"] = 1
			print("|cFFFFFF99"..CRITSOUND_HEALON)
		elseif (param == "off") then
			SoundSwitch["heal"] = 0
			print("|cFFFFFF99"..CRITSOUND_HEALOFF)
		else
			print("|cFFFFFF99"..CRITSOUND_ONOFFERROR)
		end
	elseif (cmd == "show") then
		print("|cFFFFFF99Sound mode: |cFF00EEFF"..SoundMode)
		print("|cFFFFFF99Aging time: |cFF00EEFF"..AgingTime)
	else
		print("|cFFFFFF99"..CRITSOUND_NOCMD)
	end
end
